#!/bin/bash
#
# Copyright (C) 2023 MOXA Inc. All rights reserved.
# This software is distributed under the terms of the MOXA SOFTWARE NOTICE.
# See the file LICENSE for details.
#
# Authors:
#       2023  Wilson YS Huang  <wilsonys.huang@moxa.com>

# shellcheck disable=SC2317 # Don't warn about unreachable commands in this file

# TODO: loading animation

#
# Global variables
#
export PRODUCT_NAME=""
export LINUX_DISTRO_NAME=""
export LINUX_DISTRO_RELEASE_LIFECYCLE_VER=""
export LINUX_DISTRO_ID=""

INSTL_TOP_DIR="$(dirname "$(realpath "${0}")")"
export INSTL_TOP_DIR
export INSTL_PRODUCT_DIR="${INSTL_TOP_DIR}/product.d"
export INSTL_SCRIPT_DIR="${INSTL_TOP_DIR}/scripts"
export INSTL_HOOKS_DIR="${INSTL_SCRIPT_DIR}/hook.d"
INSTL_LOG_FILE="${INSTL_TOP_DIR}/install.log"
export INSTL_LOG_FILE
INSTL_VERSION="$(cat "${INSTL_TOP_DIR}"/version)"
export INSTL_VERSION
export INSTL_LOG_LEVEL="info"
export INSTL_EXIT_CODE=0
INSTL_PROMPT_AUTO_YES=false
export INSTL_PROMPT_AUTO_YES
INSTL_OPERATION="install"
INSTL_LOCKFILE="/var/lock/moxa-sdk.lock"
INSTL_CONF_DIR="/etc/moxa"
export INSTL_CONF_DIR
INSTL_SRC_VERSION_FILE="${INSTL_CONF_DIR}/x86-src-version"
export INSTL_SRC_VERSION_FILE
INSTL_FORCE_INSTALL=false
export INSTL_FORCE_INSTALL

SUPPORT_LINUX_DISTRO=(
        "debian"
        "ubuntu"
        "rhel"
        "centos"
)

source "${INSTL_SCRIPT_DIR}/lib/util.sh"
source "${INSTL_SCRIPT_DIR}/lib/logger.sh"
source "${INSTL_SCRIPT_DIR}/lib/version.sh"

read_product_name_from_dmi_type12() {
        /usr/sbin/dmidecode -t 12 |
                grep "Option " |
                awk -F ':' '{print substr($2,1,11)}' |
                sed 's/ //g'
}

read_product_name_from_dmi_type1() {
        /usr/sbin/dmidecode -t 1 |
                grep "Product Name" |
                awk -F ':' '{print $2}' |
                sed 's/ //g'
}

read_product_name() {
        local product_name
        local dmi_info

        dmi_info="$(read_product_name_from_dmi_type12) $(read_product_name_from_dmi_type1)"

        for name in ${dmi_info}; do
                log::debug "name=${name}"
                if util::is_dir "${INSTL_PRODUCT_DIR}/${name}"; then
                        product_name="${name}"
                        break
                fi
        done

        if util::is_empty_str "${product_name}"; then
                log::error "Product is not support."
                exit 1
        fi

        PRODUCT_NAME="${product_name}"
}

load_product_config() {
        read_product_name

        if ! util::is_exists "${INSTL_PRODUCT_DIR}/${PRODUCT_NAME}/config"; then
                log::error "Could not find ${PRODUCT_NAME} configuration."
                exit 2
        fi

        source "${INSTL_PRODUCT_DIR}/${PRODUCT_NAME}/config"
}

read_os_info() {
        local option=$1

        awk -v opt="${option}" -F= '$1==opt { print $2 ;}' /etc/os-release |
                tr -d '"'
}

read_os_release_lifecycle_ver() {
        local os_id
        local os_version_id
        local ver

        os_id="$1"
        os_version_id=$(read_os_info "VERSION_ID")

        case $os_id in
        "debian") ver=$os_version_id ;;
        "ubuntu") ver=$os_version_id ;;
        "rhel") ver=$(ver::get_major "$os_version_id") ;;
        "centos") ver=$os_version_id ;;
        esac

        echo "$ver"
}

check_os_env() {
        local os_id
        local os_release_lifecycle_ver
        local os_name

        os_id=$(read_os_info "ID")
        os_release_lifecycle_ver=$(read_os_release_lifecycle_ver "$os_id")
        os_name=$(read_os_info "NAME")

        log::debug "os_id=$os_id"
        log::debug "os_release_lifecycle_ver=$os_release_lifecycle_ver"
        log::debug "os_name=$os_name"

        # 1. check os is support
        log::debug "SUPPORT_LINUX_DISTRO=${SUPPORT_LINUX_DISTRO[*]}"
        if [[ ! "${SUPPORT_LINUX_DISTRO[*]}" =~ ${os_id} ]]; then
                log::error "Linux distro \"${os_name}\" is not support."
                exit 1
        fi

        # 2. check product is support os
        log::debug "CFG_SUPPORT_LINUX_DISTRO=${CFG_SUPPORT_LINUX_DISTRO}"
        if [[ ! "${CFG_SUPPORT_LINUX_DISTRO}" =~ ${os_id} ]]; then
                log::error "Product \"${PRODUCT_NAME}\" is not support Linux distro \"${os_name}\"."
                exit 1
        fi

        LINUX_DISTRO_NAME="${os_name}"
        LINUX_DISTRO_RELEASE_LIFECYCLE_VER="${os_release_lifecycle_ver}"
        LINUX_DISTRO_ID="${os_id}"
}

exec_hook() {
        local hook_script
        local exit_code
        hook_script="${INSTL_SCRIPT_DIR}/${1}"

        if util::is_file "${hook_script}"; then
                log::info ">>> Execute hook script \"$(basename "${hook_script}")\"."
                ${hook_script}
                exit_code=${?}

                if [[ ${exit_code} -ne 0 ]]; then
                        INSTL_EXIT_CODE=${exit_code}
                fi

                log::info "<<< Execute hook script \"$(basename "${hook_script}")\" done."
        else
                log::warn "=== Execute hook script \"$(basename "${hook_script}")\" is not file."
        fi
}

process_file_list() {
        local os_version="$1"
        local repo_name="$2"
        local repo_tag="$3"
        local repo_ver

        if util::is_exists "${INSTL_TOP_DIR}"/src/"${repo_name}"-"${repo_tag}"/version; then
                repo_ver="$(cat "${INSTL_TOP_DIR}"/src/"${repo_name}"-"${repo_tag}"/version)"
        else
                repo_ver="unknown"
        fi

        if [[ "${LINUX_DISTRO_RELEASE_LIFECYCLE_VER}" == "$os_version" ]]; then
                printf " %-30s%-25s%-10s\n" "${repo_name}" "${repo_ver}" "${repo_tag}"
                return 0
        else
                return 1
        fi
}

dry_run() {
        local drivers_list
        local tools_list
        local driver_available
        local tool_available

        driver_available=true
        tool_available=true

        drivers_list="${INSTL_PRODUCT_DIR}/${PRODUCT_NAME}/drivers"
        tools_list="${INSTL_PRODUCT_DIR}/${PRODUCT_NAME}/tools"
        printf "%70s\n" " " | tr ' ' '-'
        echo " Product Name: ${PRODUCT_NAME}"
        printf "%70s\n" " " | tr ' ' '-'
        printf " %-30s%-25s%-10s\n" "Name" "Version" "Tag"
        printf "%70s\n" " " | tr ' ' '='

        if ! util::parse_product_file "${drivers_list}" "${LINUX_DISTRO_ID}" process_file_list; then
                driver_available=false
        fi

        if ! util::parse_product_file "${tools_list}" "${LINUX_DISTRO_ID}" process_file_list; then
                tool_available=false
        fi

        if [[ $driver_available = false ]] && [[ $tool_available = false ]]; then
                printf " %-30s%-25s%-10s\n" "No available" "N/A" "N/A"
        fi

        printf "%70s\n" " " | tr ' ' '-'
}

script_acquire_lock() {
        if ! (util::is_exists "${INSTL_LOCKFILE}" && kill -0 "$(cat ${INSTL_LOCKFILE})"); then
                echo $$ >"${INSTL_LOCKFILE}"
                return 0
        else
                return 1
        fi
}

script_release_lock() {
        rm -f "${INSTL_LOCKFILE}"
}

script_signal_handler() {
        script_release_lock
}

script_signal_INT_handler() {
        script_release_lock
        log::info "Interrupt!"
        exit 0
}

script_init() {
        if ! util::is_root_user; then
                echo "Permission denied. Are you root?"
                exit 13
        fi

        if ! script_acquire_lock; then
                echo "Another install.sh process is running."
                exit 16
        fi

        trap script_signal_handler ERR TERM EXIT
        trap script_signal_INT_handler INT

        if util::is_exists "${INSTL_LOG_FILE}"; then
                echo >"${INSTL_LOG_FILE}"
        else
                touch "${INSTL_LOG_FILE}"
        fi

        log::init "${INSTL_LOG_FILE}" "${INSTL_LOG_LEVEL}"
}

script_help_page() {
        cat <<EOF

                ███╗   ███╗ ██████╗ ██╗  ██╗ █████╗
                ████╗ ████║██╔═══██╗╚██╗██╔╝██╔══██╗
                ██╔████╔██║██║   ██║ ╚███╔╝ ██║  ██║
                ██║╚██╔╝██║██║   ██║ ██╔██╗ ██║  ██║
                ██║ ╚═╝ ██║╚██████╔╝██╔╝ ██╗██║  ██║
                ╚═╝     ╚═╝ ╚═════╝ ╚═╝  ╚═╝╚═╝  ╚═╝
                ------------------------------------
                         X86 INSTALL WIZARD

Usage: install.sh [option]

Options:
        -h, --help         Display this help page
        -y, --yes          Automatically answer yes
        -v, --version      Display the version information
        -s, --selftest     Run the self test cases
            --dry-run      List available driver and tool only
            --uninstall    Uninstall driver and tool
            --force        Install driver and tool even if the
                           version is the same or older (default is
                           to install newer version)

Without passing any option, it would run the installation directly.
EOF
}

script_params() {
        local param

        while [[ $# -gt 0 ]]; do
                param="$1"
                shift
                case "$param" in
                -h | --help)
                        script_help_page
                        exit 0
                        ;;
                -y | --yes)
                        INSTL_PROMPT_AUTO_YES=true
                        ;;
                -v | --version)
                        echo "${INSTL_VERSION}"
                        exit 0
                        ;;
                -s | --selftest)
                        INSTL_OPERATION="selftest"
                        ;;
                --dry-run)
                        INSTL_OPERATION="dryrun"
                        ;;
                --uninstall)
                        INSTL_OPERATION="uninstall"
                        ;;
                --force)
                        INSTL_FORCE_INSTALL=true
                        ;;
                *)
                        echo "Invalid option was provided: $param"
                        script_help_page
                        exit 22
                        ;;
                esac
        done
}

show_system_info() {
        log::info "Product Name: ${PRODUCT_NAME}"
        log::info "OS Name: ${LINUX_DISTRO_NAME}"
        log::info "OS Version: ${LINUX_DISTRO_RELEASE_LIFECYCLE_VER}"
        log::info "Kernel Info: $(uname -a)"
}

main() {
        script_params "$@"
        script_init
        load_product_config
        check_os_env

        case "${INSTL_OPERATION}" in
        "install")
                show_system_info
                if ! util::confirm_prompt "Do you want to continue?" "${INSTL_PROMPT_AUTO_YES}"; then
                        log::info "Abort"
                        exit 0
                fi

                exec_hook "install-dev-tools.sh"
                if [[ "$INSTL_EXIT_CODE" -ne 0 ]]; then
                        log::error "Failed, exit code: ${INSTL_EXIT_CODE}. See log at install.log for details."
                        exit ${INSTL_EXIT_CODE}
                fi

                exec_hook "build-and-install-source.sh"
                if [[ "$INSTL_EXIT_CODE" -eq 0 ]]; then
                        exec_hook "self-test.sh"
                        log::info "Done. Please reboot machine for installation to take effect."

                        if util::confirm_prompt "Do you want to reboot now?" false; then
                                systemctl reboot
                        fi
                        exit 0
                elif [[ "$INSTL_EXIT_CODE" -eq 3 ]]; then
                        exec_hook "self-test.sh"
                        log::info "Done. Nothing was installed"
                        exit 0
                else
                        log::error "Failed, exit code: ${INSTL_EXIT_CODE}. See log at install.log for details."
                        exit ${INSTL_EXIT_CODE}
                fi
                ;;
        "uninstall")
                show_system_info
                if ! util::confirm_prompt "Do you want to continue?" "${INSTL_PROMPT_AUTO_YES}"; then
                        log::info "Abort"
                        exit 0
                fi

                exec_hook "uninstall.sh"

                if [[ "$INSTL_EXIT_CODE" -eq 0 ]]; then
                        log::info "Done. Please reboot machine for uninstallation to take effect."

                        if util::confirm_prompt "Do you want to reboot now?" false; then
                                systemctl reboot
                        fi
                        exit 0
                elif [[ "$INSTL_EXIT_CODE" -eq 3 ]]; then
                        log::info "Done. Nothing was uninstalled"
                        exit 0
                else
                        log::error "Failed, exit code: ${INSTL_EXIT_CODE}. See log at install.log for details."
                        exit ${INSTL_EXIT_CODE}
                fi
                ;;
        "selftest")
                show_system_info
                exec_hook "self-test.sh"
                exit 0
                ;;
        "dryrun")
                dry_run
                exit 0
                ;;
        *)
                log::error "Unknown operation: ${INSTL_OPERATION}"
                exit 1
                ;;
        esac

        script_release_lock
}

main "$@"
